#!/usr/bin/python

import re
import qubtree
import operator
from itertools import *
import traceback
from math import *

from Dialogs import acceptFloatList

def float_or_def(s, d):
    try:
        return float(s)
    except:
        return d

line_re = re.compile(r"\s*(\S+)\s+(\S+)\s+(\S+).*")
burst_re = re.compile(r"BURST")
amp_re = re.compile(r"Mean amp.*=\s*([-0-9eE+.]+)\s+$")

def read_ekdist_burst(fi):
    """Returns durations, amps, mean_open_amp, including final too-long closure"""
    durs, amps = [], []
    mean = -1.0
    while True:
        try:
            line = fi.next()
            if burst_re.match(line):
                break
            match = line_re.match(line)
            if match:
                durs.append(float(match.group(2)))
                amps.append(float(match.group(3)))
            elif line.strip():
                return [], [], -1.0 # this is not a burst
        except StopIteration:
            if durs:
                break # make a segment out of the end of file
            else:
                raise
        except ValueError:
            return [], [], -1.0 # this is not a burst

    match = amp_re.match(fi.next())
    if match:
        mean = float_or_def(match.group(1), mean)
    fi.next()
    fi.next()
    fi.next()

    return durs[:-1], amps[:-1], mean

def join_neighboring(durations, ampers):
    # assuming 0.0 amp is never repeated, and all other amps are of same class
    durs, amps = [], []
    D = A = 0.0
    for d, a in izip(durations, ampers):
        if a == 0.0: # end opening
            if D:
                durs.append(D)
                amps.append(A/D)
                D = A = 0.0
            durs.append(d)
            amps.append(a)
        else:
            D += d
            A += d*a
    if D:
        durs.append(D)
        amps.append(A/D)
    return durs, amps

def classify_threshold(Acrit, durs, amps):
    if len(Acrit) == 0:
        return durs, [a and 1 or 0 for a in amps], amps
    elif len(Acrit) == 1:
        thresh = Acrit[0]
        if thresh >= 0.0:
            op = operator.ge
        else:
            op = operator.le
        return durs, [op(a, thresh) and 1 or 0 for a in amps], amps
    # else (len > 1):
    thresh = Acrit[:]
    clss = []
    thresh.sort()
    ncls = len(thresh)
    if abs(thresh[0]) > abs(thresh[-1]):
        thresh.reverse()
        op = operator.gt
    else:
        op = operator.lt
    for a in amps:
        for c,t in enumerate(thresh):
            if op(a, t):
                clss.append(c)
                break
        else: # that's right: for-else (when exited naturally not via break)
            clss.append(ncls)
    return durs, clss, amps

def measure_amps(durs, clss, amps):
    ncls = max(clss) + 1
    cls_amp = [0.0] * ncls
    cls_wt  = [0.0] * ncls
    for i in xrange(len(durs)):
        c = clss[i]
        cls_amp[c] += amps[i] * durs[i]
        cls_wt[c]  += durs[i]
    for c in xrange(ncls):
        if cls_wt[c]:
            cls_amp[c] /= cls_wt[c]
    return durs, clss, cls_amp

def burst_to_segment(durs, amps, mean, Acrit, start):
    durs, clss, cls_amp = measure_amps(*classify_threshold(Acrit, *join_neighboring(durs[:-1], amps[:-1])))
    seg = qubtree.Node('Segment')
    seg['start'].data = start
    seg['amp'].data = cls_amp
    seg['sd'].data = [0.1] * len(cls_amp)
    seg['DwellCount'].data = len(durs)
    seg['Classes'].data = clss
    durations = seg['Durations'].data
    durations.setup(qubtree.QTR_TYPE_FLOAT, len(durs), 1)
    for i in xrange(len(durs)):
        durations[i] = durs[i]
    return seg

def ReadPRT(path, Acrit=''):
    dataset = qubtree.Node('DataSet')
    seg = qubtree.NullNode()
    start = 0.0
    try:
        A_crit = acceptFloatList()(Acrit)
    except:
        traceback.print_exc()
        A_crit = []
    try:
        fi = file(path, "r")
        while True:
            durs, amps, mean = read_ekdist_burst(fi)
            if 0 == len(durs):
                continue
            seg = dataset.insert(burst_to_segment(durs, amps, mean, A_crit, start), seg)
            start += sum(durs)
    except StopIteration:
        pass
    except:
        traceback.print_exc()
    return dataset

if __name__ == "__main__":
    import sys
    try:
        dataset = ReadPRT(sys.argv[1])
        dataset.saveAs(sys.argv[2])
    except IndexError:
        print dataset
    except:
        traceback.print_exc()

